/* Copyright (c) 2000-2009 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

/**
 * A goto defintion hyperlink handler.
 *
 * @class
 * @base ko.hyperlinks.BaseHandler
 */
ko.hyperlinks.GotoDefinitionHandler = function() {
    var name = "Goto Defintion";
    var fn = function(args) {
        ko.views.manager.do_cmd_goToDefinition.apply(null, args);
    };
    var lang_names = null;  /* All language types */
    var indic_style = Components.interfaces.ISciMoz.INDIC_PLAIN;
    //var indic_color = RGB(0xff,0x80,0x20);
    var indic_color = RGB(0xA0,0x00,0xF0);
    var base_args = [name, fn, lang_names, indic_style, indic_color];
    ko.hyperlinks.BaseHandler.apply(this, base_args);
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
ko.hyperlinks.GotoDefinitionHandler.prototype = new ko.hyperlinks.BaseHandler();
ko.hyperlinks.GotoDefinitionHandler.prototype.constructor = ko.hyperlinks.GotoDefinitionHandler;

/**
 * Try and show a hyperlink at the current position in the view.
 *
 * @param view {Components.interfaces.koIScintillaView}  View to check.
 * @param scimoz {Components.interfaces.ISciMoz}  Scimoz for the view.
 * @param pos {int}  Position in the scimoz editor.
 * @param line {string}  The current line from the editor.
 * @param lineStartPos {int} Scimoz position for the start of the line.
 * @param lineEndPos {int}   Scimoz position for the end of the line.
 * @returns {ko.hyperlinks.Hyperlink} - The hyperlink instance shown.
 */
ko.hyperlinks.GotoDefinitionHandler.prototype.show = function(
                view, scimoz, position, line, lineStartPos, lineEndPos)
{
    // For goto definition, if this view does not support codeintel
    // citadel stuff, then goto definition is not supported and
    // there is no need to mark any hyperlinks.
    if (!view.isCICitadelStuffEnabled) {
        return null;
    }

    var start = scimoz.wordStartPosition(position, true);
    var end = scimoz.wordEndPosition(position, true);
    if (start < end) {
        var next_start;
        while (1) {
            next_start = scimoz.wordStartPosition(start, false);
            if (next_start < start) {
                // XXX: This needs to use language specific separators.
                if ([".", "->", "::"].indexOf(scimoz.getTextRange(next_start, start)) >= 0) {
                    start = scimoz.wordStartPosition(next_start, true);
                    continue;
                }
            }
            break;
        }
        return this.setHyperlink(view, start, end);
    }
    return null;
}

ko.hyperlinks.addHandler(new ko.hyperlinks.GotoDefinitionHandler());
